-- Copyright (c) 2024 - Ognjen 'xolatile' Milan Robovic
--
-- GNU General Public Licence (version 3 or later)

with core, ui, effect, magic;

package body magic is

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

	view_width  : constant natural := 90;
	view_height : constant natural := 90;

	view_sprite : array (enumeration) of core.sprite;
	icon_sprite : array (enumeration) of core.sprite;

                                             ------------------------------------------------------------------------------------------

	procedure configure is
	begin
		core.echo (core.comment, "Configuring magic components...");
		--
		for index in enumeration
		loop
			declare
				folder : constant string := core.lowercase (school'image (trait (index).kind));
				file   : constant string := core.lowercase (enumeration'image (index));
			begin
				view_sprite (index) := core.import_sprite ("./sprite/magic/" & folder & "/" & file & ".png", 1, 1);
				icon_sprite (index) := core.import_sprite ("./sprite/magic/icon/"           & file & ".png", 1, 1);
			end;
		end loop;
	end configure;

                                             ------------------------------------------------------------------------------------------

	procedure icon (index : in enumeration; x, y : in integer) is
	begin
		core.draw        (icon_sprite (index), x + 2, y + 2);
		ui.draw_overicon (trait (index).text, x, y);
	end icon;

                                             ------------------------------------------------------------------------------------------

	procedure view (index : in enumeration; x, y : in integer) is
		offset : constant integer := 4;
	begin
		core.draw         (view_sprite (index), x + offset, y + offset);
		ui.draw_icon_menu (trait (index).text, x, y, view_width + 2 * offset, view_height + 2 * offset);
	end view;

                                             ------------------------------------------------------------------------------------------

	procedure menu (x, y : in integer; center : in boolean) is
		offset : constant integer := 16;
		width  : constant integer := 168 * 8         + 2 * offset;
		height : constant integer :=  12 * core.icon + 2 * offset;
		move_x : constant integer := (if center then (core.window_width  - width)  / 2 else x);
		move_y : constant integer := (if center then (core.window_height - height) / 2 else y);
		--
		next : array (0 .. 7) of integer := (others => 0);
	begin
		ui.draw_tiny_menu (move_x, move_y, width, height, true);
		ui.draw_title_bar (move_x, move_y, width, "Magic Abilities");
		--
		for index in enumeration
		loop
			icon     (index,              move_x + offset + school'pos (trait (index).kind) * 168,             move_y + offset + next (school'pos (trait (index).kind)) * core.icon);
			ui.write (trait (index).name, move_x + offset + school'pos (trait (index).kind) * 168 + core.icon, move_y + offset + next (school'pos (trait (index).kind)) * core.icon);
			--
			next (school'pos (trait (index).kind)) := next (school'pos (trait (index).kind)) + 1;
		end loop;
	end menu;

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

end magic;
