-- Copyright (c) 2024 - Ognjen 'xolatile' Milan Robovic
--
-- GNU General Public Licence (version 3 or later)

pragma ada_2012;

with core, ui, effect, attribute, skill, resource, faction, might, magic, item, unit, construction, chad, world, ai;

procedure main is

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

	side_panel     : integer := 480;
	preview_x      : integer := 24;
	preview_y      : integer := 24;
	preview_width  : integer := 0;
	preview_height : integer := 0;

	player : chad.information := chad.trait (chad.ognjen);

                                             ------------------------------------------------------------------------------------------

	type menu_index is (
		menu_none, menu_attribute, menu_skill, menu_resource, menu_unit, menu_might, menu_magic
	);

	menu_limit : constant integer := 3;
	menu_count :          integer := 0;

	menu_stack : array (1 .. menu_limit) of menu_index := (others => menu_none);

	procedure menu_insert (index : in menu_index) is
	begin
		if menu_count = menu_limit then return; end if;
		--
		menu_count := menu_count mod menu_limit + 1;
		--
		menu_stack (menu_count) := index;
	end menu_insert;

	procedure menu_remove is
	begin
		if menu_count = 0 then return; end if;
		--
		menu_stack (menu_count) := menu_none;
		--
		menu_count := menu_count - 1;
	end menu_remove;

	procedure menu_render is
	begin
		if menu_count > 0 then
			core.overlay;
		end if;
		--
		for index in 1 .. menu_limit
		loop
			case menu_stack (index) is
				when menu_none      => null;
				when menu_attribute => attribute.menu (100, 100, false);
				when menu_skill     => skill.menu     (200, 200, false);
				when menu_resource  => resource.menu  (300, 300, false);
				when menu_unit      => unit.menu      (0,   0,   true);
				when menu_might     => might.menu     (0,   0,   true);
				when menu_magic     => magic.menu     (0,   0,   true);
			end case;
		end loop;
	end menu_render;

                                             ------------------------------------------------------------------------------------------

	procedure idle                is begin null;                               end idle;
	procedure move_camera_up      is begin core.camera.y := core.camera.y - 1; end move_camera_up;
	procedure move_camera_down    is begin core.camera.y := core.camera.y + 1; end move_camera_down;
	procedure move_camera_left    is begin core.camera.x := core.camera.x - 1; end move_camera_left;
	procedure move_camera_right   is begin core.camera.x := core.camera.x + 1; end move_camera_right;
	procedure show_attribute_menu is begin menu_insert (menu_attribute);       end show_attribute_menu;
	procedure show_skill_menu     is begin menu_insert (menu_skill);           end show_skill_menu;
	procedure show_resource_menu  is begin menu_insert (menu_resource);        end show_resource_menu;
	procedure show_unit_menu      is begin menu_insert (menu_unit);            end show_unit_menu;
	procedure show_might_menu     is begin menu_insert (menu_might);           end show_might_menu;
	procedure show_magic_menu     is begin menu_insert (menu_magic);           end show_magic_menu;
	procedure hide_top_menu       is begin menu_remove;                        end hide_top_menu;

	signal_list : constant array (core.signal_code) of access procedure := (
		core.signal_up    => move_camera_up'access,
		core.signal_down  => move_camera_down'access,
		core.signal_left  => move_camera_left'access,
		core.signal_right => move_camera_right'access,
		core.signal_a     => show_attribute_menu'access,
		core.signal_s     => show_skill_menu'access,
		core.signal_r     => show_resource_menu'access,
		core.signal_u     => show_unit_menu'access,
		core.signal_m     => show_might_menu'access,
		core.signal_n     => show_magic_menu'access,
		core.signal_grave => hide_top_menu'access,
		others            => idle'access
	);

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

begin

	core.dash;
	core.echo (core.comment, "Copyright (C) 2024 -- Ognjen 'xolatile' Milan Robovic");
	core.echo (core.comment, "Version -- 1.0.0");
	core.echo (core.comment, "License -- GNU/GPLv3+");
	core.echo (core.comment, "Xhads is free software, you can redistribute it and modify it under the terms of the GNU General Public License by Free Software Foundation.");
	core.dash;

	core.initialize;

	ui.configure;

	core.play_song (core.import_song (core.c_string ("./song/main_menu.ogg")));

	attribute.configure;
	skill.configure;
	resource.configure;
	--~might.configure;
	--~magic.configure;
	item.configure;
	unit.configure;
	construction.configure;
	chad.configure;
	world.configure;
	ai.configure;

	world.make (world.grass, 120, 100);

	preview_width  := core.window_width - side_panel;
	preview_height := core.window_height;

	core.dash;
	core.echo (core.success, "Successfully initialized game data, entering main gameplay loop.");
	core.dash;

                                             ------------------------------------------------------------------------------------------

	gameplay: loop
		core.synchronize;
		--
		exit when core.engine_active = false;
		--
		ui.active := (if core.cursor_mode = 3 then ui.default else ui.steam);
		--
		core.camera.x := core.clip (core.camera.x, 0, world.map.width  - preview_width  / core.base);
		core.camera.y := core.clip (core.camera.y, 0, world.map.height - preview_height / core.base);
		--
		world.draw (preview_x, preview_y, preview_width - 2 * preview_x, preview_height - 2 * preview_y - 32, core.signal_mode = core.signal_code'pos (core.signal_g));
		--
		--~core.draw_central_grid (preview_x, preview_y, preview_width - 2 * preview_x, preview_height - 2 * preview_y);
		--~core.draw_squared_grid (preview_x, preview_y, preview_width - 2 * preview_x, preview_height - 2 * preview_y);
		--~core.draw_hexagon_grid (preview_x, preview_y, preview_width - 2 * preview_x, preview_height - 2 * preview_y);
		--
		ui.draw_menu      (0,             0, preview_width, preview_height - 32, false);
		ui.draw_tiny_menu (preview_width, 0, side_panel,    preview_height - 32, true);
		--
		ui.draw_state_box (preview_width + 32, 32);
		--
		signal_list (core.signal_code'val (core.signal_mode)).all;
		--
		--~magic.menu (0, 0, true);
		--~might.menu (0, 0, true);
		--
		menu_render;
		--
		ui.write (menu_count'image, 16, 16);
		--
		ui.draw_text_box (0, core.window_height - 32, core.window_width, 32);
	end loop gameplay;

                                             ------------------------------------------------------------------------------------------

	core.deinitialize;

	core.dash;

end main;
